﻿/*
*
* Copyright (C) 2007 Mike Blamires
* All Rights Reserved
* Released under GPL2.0 - please see attached document.
*
* http://blog.blamires.co.uk/
* mike@blamires.co.uk
*
* Version 0.1 beta
*
* Bugs:
*  > Leaves the name of the account in the textbox after unlocking.
*
* TODO:
*  > Refactor
*  > seperate UI and AD functions
* 
*/

using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;

using System.DirectoryServices;
using System.Threading;

namespace AccountUnlock {
    public partial class Form1 : Form {

	//Delegates for thread safe form control.
        delegate void ClearItemsD();
        delegate void AddItemD(String aString);
        delegate void SelectFirstEntryD();

        private static Thread t;

        public Form1() {

            InitializeComponent();

            textBox1.Text = Settings1.Default.LDAPPath;
            numericUpDown1.Value = new Decimal(Settings1.Default.CheckInterval);
            if (Settings1.Default.Alert) {
                checkBox1.Checked = true;
            }

            this.ShowInTaskbar = true;
            notifyIcon1.Click += new System.EventHandler(this.notifyIcon1_DoubleClick);
            notifyIcon1.Visible = true;

            getLockedAccounts();

            t = new Thread(new ThreadStart(ThreadProc));
            t.Start();

        }

	//the core of the thread, 
        public void ThreadProc() {
            try {
                while (true) {
                    clearItems();
                    getLockedAccounts();
                    Thread.Sleep(Settings1.Default.CheckInterval * 60 * 1000); //checks
                }
            } catch (ThreadAbortException tae) { //just catch
            } catch (Exception e) {
                //feedback failure and log the exception.
                using (System.Diagnostics.EventLog el = new System.Diagnostics.EventLog()) {
                    MessageBox.Show("Error: Please see the EventLog Application Log");
                    el.Source = "AccountUnlock"; //this will show where the error originates from
                    el.WriteEntry(e.ToString()); //writes the content of the entry.
                }
            }
        }

        //thread safe methods for adjusting the form.
        private void clearItems() {
            try {
                if (this.comboBox1.InvokeRequired) {
                    ClearItemsD d = new ClearItemsD(clearItemsMethod);
                    this.Invoke(d);
                } else {
                    comboBox1.Items.Clear();
                }
            } catch (ObjectDisposedException) { }
        }

        private void addItem(String aString) {
            try {
                if (this.comboBox1.InvokeRequired) {
                    AddItemD d = new AddItemD(addItemMethod);
                    this.Invoke(d, new object[] { aString });
                }
            } catch (ObjectDisposedException) { }
        }

        private void selectFirstEntry() {
            try {
                if (this.comboBox1.InvokeRequired) {
                    SelectFirstEntryD d = new SelectFirstEntryD(selectFirstEntryMethod);
                    this.Invoke(d);
                }
            } catch (ObjectDisposedException) { }
        }

        private void addItemMethod(String name) {
            comboBox1.Items.Add(name);
        }

        private void clearItemsMethod() {
            comboBox1.Items.Clear();
            comboBox1.Text = "";
        }

        private void selectFirstEntryMethod() {
            comboBox1.SelectedIndex = 0;
        }
        //End threadsafe for access methods.


		
        private void notifyIcon1_DoubleClick(object sender, EventArgs e) {
            this.Visible = !this.Visible; //show if hidden, hide if shown.
            this.ShowInTaskbar = !this.ShowInTaskbar; //show in taskbar if not in taskbar, hide from taskbar if in taskbar
            this.Activate();
	}

	//Closes the thread and exits the application.
        private void button3_Click(object sender, EventArgs e) {
            t.Abort();
            t = null;
            Application.Exit();
        }

	//refresh the list of accounts
        private void button2_Click(object sender, EventArgs e) {
            clearItems();
            getLockedAccounts();
        }


	//enumerates through all the accounts locked out in the LDAP path and below.
        private void getLockedAccounts() {
            try {
                //The "(objectClass=user))(lockoutTime>=1)" defines a locked out user, any account picked up 
                //will be locked.
                string filter = "(&(&(objectCategory=person)(objectClass=user))(lockoutTime>=1))";
                string[] propertiesToLoad = new string[2] { "name", "lockoutTime" };

                using (DirectoryEntry root = new DirectoryEntry(Settings1.Default.LDAPPath))
                using (DirectorySearcher searcher = new DirectorySearcher(root, filter, propertiesToLoad))
                using (SearchResultCollection results = searcher.FindAll()) {
                    //we must loop through locked out accounts as there may be several
                    foreach (SearchResult result in results) {
                        try {
                            addItem((string)result.Properties["name"][0]);
                        } catch (ArgumentOutOfRangeException) { }
                    }
                    try {
                        selectFirstEntryMethod(); //form1
                    } catch { }
                }
                if (Settings1.Default.Alert) {
                    if (comboBox1.Items.Count > 0) {
                        notifyIcon1.ShowBalloonTip(5000, "Locked Accounts", "There are " + comboBox1.Items.Count.ToString() + " locked out accounts.", ToolTipIcon.Info);
                    }
                }
            } catch (Exception ex) {
                using (System.Diagnostics.EventLog el = new System.Diagnostics.EventLog()) {
                    MessageBox.Show("Error: Please see the EventLog Application Log");
                    el.Source = "AccountUnlock"; //this will show where the error originates from
                    el.WriteEntry(ex.ToString()); //writes the content of the entry.
                }
            }
        }

	//saves the 3 user settings in the account 
        private void button5_Click(object sender, EventArgs e) {
            Settings1.Default.LDAPPath = @textBox1.Text;
            Settings1.Default.CheckInterval = (int)numericUpDown1.Value;
            if (checkBox1.Checked) {
                Settings1.Default.Alert = true;
            } else {
                Settings1.Default.Alert = false;
            }
            Settings1.Default.Save();

        }

	//initiates an unlock of all accounts
        private void button4_Click(object sender, EventArgs e) {
            if (MessageBox.Show("Are you sure you want to unlock all the accounts?", "Warning", MessageBoxButtons.OKCancel, MessageBoxIcon.Question) == DialogResult.OK)
                for (int i = 0; i < comboBox1.Items.Count; i++)
                    unlock(comboBox1.Items[i].ToString());
        }

	//initiates an unlock of the selected account
	private void button1_Click(object sender, EventArgs e) {
            try {
                unlock(comboBox1.SelectedItem.ToString());
            } catch (NullReferenceException nre) { 
                //don't do anything with no item selected.
            }
            clearItems();
            getLockedAccounts();
        }

	//unlocks an account as given by "name".
        private void unlock(String name) {
            try {
                //Search for our directory object and set the "lockoutTime" property to 0x0000 to 
                //unlock the account.
                string filter = "(name=" + name + ")";//
                string[] propertiesToLoad = new string[1] { "lockoutTime" };
                DirectoryEntry user = new DirectoryEntry(Settings1.Default.LDAPPath);
                DirectorySearcher searcher = new DirectorySearcher(user, filter, propertiesToLoad);
                SearchResult result = searcher.FindOne();
                user = result.GetDirectoryEntry();
                user.Properties["lockoutTime"].Value = 0x0000;
                user.CommitChanges();
                //feedback success
            } catch (Exception ex) {
                //feedback failure and log the exception.
                using (System.Diagnostics.EventLog el = new System.Diagnostics.EventLog()) {
                    MessageBox.Show("Error: Please see the EventLog Application Log");
                    el.Source = "AccountUnlock"; //this will show where the error originates from
                    el.WriteEntry(ex.ToString()); //writes the content of the entry.
                    return;
                }
            }
            MessageBox.Show(String.Format("{0} - account unlocked", name));
        }
    }
}
